@@ -2,10 +2,18 @@ import datetime
 import json
 from abc import abstractmethod
 from decimal import Decimal
-from typing import Any, Coroutine, Iterable, List, Optional
+from typing import Any, Coroutine, Iterable, List, Optional, Tuple
 
 from starlette.requests import Request
-from tortoise import ForeignKeyFieldInstance, ManyToManyFieldInstance, Model, fields
+from tortoise import (
+    BackwardFKRelation,
+    BackwardOneToOneRelation,
+    ForeignKeyFieldInstance,
+    ManyToManyFieldInstance,
+    Model,
+    OneToOneFieldInstance,
+    fields,
+)
 from tortoise.fields.data import CharEnumFieldInstance, IntEnumFieldInstance
 
 from fast_tmp.amis.actions import DialogAction
@@ -31,10 +39,16 @@ from fast_tmp.contrib.auth.hashers import make_password
 from fast_tmp.contrib.tortoise.fields import FileField, ImageField, RichTextField
 from fast_tmp.exceptions import TmpValueError
 from fast_tmp.responses import ListDataWithPage
-from fast_tmp.site.base import BaseAdminControl
+from fast_tmp.site.base import BaseAdminControl, BaseControl
 from fast_tmp.utils import add_media_start, remove_media_start
 
 
+class PkControl(BaseAdminControl):
+    """
+    主键
+    """
+
+
 class StrControl(BaseAdminControl):
     """
     基础的字符串control
@@ -62,12 +76,12 @@ class DecimalControl(BaseAdminControl):
             self._control = NumberItem(
                 type=self._control_type, name=self.name, label=self.label, big=True
             )
-            self._control.precision = self._field.decimal_places  # type: ignore
-            self._control.max = self._field.max_digits * 9  # type: ignore
-            if not self._field.null:  # type: ignore
+            self._control.precision = self._field.decimal_places
+            self._control.max = self._field.max_digits * 9
+            if not self._field.null:
                 self._control.required = True
-            if self._field.default is not None:  # type: ignore
-                self._control.value = self.orm_2_amis(self._field.default)  # type: ignore
+            if self._field.default is not None:
+                self._control.value = self.orm_2_amis(self._field.default)
         return self._control
 
     def amis_2_orm(self, value: float) -> Any:
@@ -87,7 +101,7 @@ class IntEnumControl(BaseAdminControl):
             super().get_formitem(request, codenames)
             d = self._control.dict(exclude_none=True)
             d.pop("type")
-            if self._field.null:  # type: ignore
+            if self._field.null:
                 d["clearable"] = True
             self._control = SelectItem(**d)
             self._control.options = self.options()
@@ -98,16 +112,16 @@ class IntEnumControl(BaseAdminControl):
             return value.name
 
     def amis_2_orm(self, value: Any) -> Any:
-        if value is None and self._field.null:  # type: ignore
+        if value is None and self._field.null:
             return None
-        for i in self._field.enum_type:  # type: ignore
+        for i in self._field.enum_type:
             if i.name == value:
                 return i.value
         raise TmpValueError(f"{self.label} 不能为 {value}")
 
     def options(self) -> List[str]:
         res = []
-        for i in self._field.enum_type:  # type: ignore
+        for i in self._field.enum_type:
             res.append(i.name)
         return res
 
@@ -118,11 +132,8 @@ class BooleanControl(IntEnumControl):
     def options(self) -> List[str]:
         return ["True", "False"]
 
-    async def validate(self, value: Any) -> Any:
-        return self.amis_2_orm(value)
-
     def amis_2_orm(self, value: Any) -> Any:
-        if (value == "None" or not value) and self._field.null:  # type: ignore
+        if not value and self._field.null:
             return None
         if value == "True":
             return True
@@ -132,7 +143,7 @@ class BooleanControl(IntEnumControl):
 
     def orm_2_amis(self, value: Any) -> Any:
         if value is None:
-            return "None"
+            return
         elif value:
             return "True"
         return "False"
@@ -157,19 +168,20 @@ class DateTimeControl(BaseAdminControl):
             self._column_inline.quickEdit.format = "YYYY-MM-DD HH:mm:ss"
         return self._column_inline
 
-    def amis_2_orm(self, value: Any) -> Any:
-        if (value == "None" or not value) and self._field.null:  # type: ignore
+    def amis_2_orm(self, value: str) -> Any:
+        if value:
+            return datetime.datetime.strptime(value, "%Y-%m-%d %H:%M:%S")
+        if self._field.null:
             return None
-        return datetime.datetime.strptime(value, "%Y-%m-%d %H:%M:%S")
+        raise TmpValueError(f"{self.label} 不能为 {value}")
 
     def orm_2_amis(self, value: datetime.datetime) -> Any:
-        if value is None:
-            return None
-        return value.strftime("%Y-%m-%d %H:%M:%S")
+        if value is not None:
+            return value.strftime("%Y-%m-%d %H:%M:%S")
 
 
 class DateControl(BaseAdminControl):
-    _control_type = FormItemEnum.date
+    _control_type = FormItemEnum.input_date
 
     def get_formitem(self, request: Request, codenames: Iterable[str]) -> FormItem:
         if not self._control:
@@ -184,10 +196,12 @@ class DateControl(BaseAdminControl):
         return self._column_inline
 
     def amis_2_orm(self, value: Any) -> Any:
-        if (value == "None" or not value) and self._field.null:  # type: ignore
+        if value:
+            year, month, day = value.split("-")
+            return datetime.date(int(year), int(month), int(day))
+        if self._field.null:
             return None
-        year, month, day = value.split("-")
-        return datetime.date(int(year), int(month), int(day))
+        raise TmpValueError(f"{self.label} 不能为 {value}")
 
     def orm_2_amis(self, value: datetime.date) -> Any:
         if value is None:
@@ -196,7 +210,7 @@ class DateControl(BaseAdminControl):
 
 
 class TimeControl(BaseAdminControl):
-    _control_type = FormItemEnum.time
+    _control_type = FormItemEnum.input_time
 
     def get_formitem(self, request: Request, codenames: Iterable[str]) -> FormItem:
         if not self._control:
@@ -213,19 +227,18 @@ class TimeControl(BaseAdminControl):
         return self._column_inline
 
     def amis_2_orm(self, value: Any) -> Any:
-        if not value:
+        if value:
+            return datetime.time.fromisoformat(value)
+        if self._field.null:
             return None
-        return datetime.time.fromisoformat(value)
+        raise TmpValueError(f"{self.label} 不能为 {value}")
 
     def orm_2_amis(self, value: Optional[datetime.time]) -> Any:
-        if value is None:
-            return None
-        if callable(value):
-            return value().strftime("%H:%M:%S")
-        return value.strftime("%H:%M:%S")
+        if value is not None:
+            return value.strftime("%H:%M:%S")
 
 
-class JsonControl(TextControl):  # fixme 用代码编辑器重构？
+class JsonControl(TextControl):
     def amis_2_orm(self, value: Any) -> Any:
         if not value:
             return None
@@ -238,6 +251,10 @@ class JsonControl(TextControl):  # fixme 用代码编辑器重构？
         if not self._control:
             super().get_formitem(request, codenames)
             self._control.validations = "isJson"
+            if not callable(self._field.default):
+                self._control.required = True  # jsonfield不能为空不然写数据库必爆炸
+                if not self._control.value:  # 如果有默认值则已经在父类里面被调用
+                    self._control.value = "{}"
         return self._control
 
     def get_column_inline(self, request: Request) -> Column:
@@ -259,7 +276,7 @@ class RelationSelectApi:
         pk: Optional[str],
         perPage: Optional[int],
         page: Optional[int],
-        filter: Any = None,
+        filter: Any = None,  # todo 考虑支持页面选项搜索
     ) -> List[dict]:
         """
         多对多或多对一的时候，用于选择项的读取
@@ -268,6 +285,12 @@ class RelationSelectApi:
 
 class ForeignKeyControl(BaseAdminControl, RelationSelectApi):
     _control_type = FormItemEnum.select
+    need_perms: Optional[Tuple[str, ...]] = None
+    _control: SelectItem = None  # type: ignore
+
+    def related_prefix(self) -> str:
+        # todo: 增加到文档，创建按钮根据页面注册的类的prefix进行搜索。
+        return self.field.related_model.__name__.lower()
 
     async def get_selects(
         self,
@@ -277,23 +300,23 @@ class ForeignKeyControl(BaseAdminControl, RelationSelectApi):
         page: Optional[int],
         filter: Any = None,
     ):
-        field_model_all = self._field.related_model.all()  # type: ignore
-        if filter:
-            if isinstance(filter, dict):
-                field_model_all = self._field.related_model.filter(**filter)  # type: ignore
-            else:
-                field_model_all = self._field.related_model.filter(filter)  # type: ignore
-        if perPage is not None and page is not None:
-            field_model = field_model_all.limit(perPage).offset((page - 1) * perPage)
-            count = await field_model_all.count()
-            data = await field_model
-            return ListDataWithPage(
-                total=count,
-                items=[{"value": i.pk, "label": str(i)} for i in data],
-            )
-        else:
-            data = await field_model_all
-            return {"options": [{"value": i.pk, "label": str(i)} for i in data]}
+        field_model_all = self._field.related_model.all()
+        # if filter:
+        #     if isinstance(filter, dict):
+        #         field_model_all = self._field.related_model.filter(**filter)
+        #     else:
+        #         field_model_all = self._field.related_model.filter(filter)
+        # if perPage is not None and page is not None:
+        #     field_model = field_model_all.limit(perPage).offset((page - 1) * perPage)
+        #     count = await field_model_all.count()
+        #     data = await field_model
+        #     return ListDataWithPage(
+        #         total=count,
+        #         items=[{"value": i.pk, "label": str(i)} for i in data],
+        #     )
+        # else:
+        data = await field_model_all
+        return {"options": [{"value": i.pk, "label": str(i)} for i in data]}
 
     def prefetch(self) -> Optional[str]:
         return "select"
@@ -312,18 +335,32 @@ class ForeignKeyControl(BaseAdminControl, RelationSelectApi):
         raise AttributeError("foreignkey field can not be used in column inline.")
 
     def get_formitem(self, request: Request, codenames: Iterable[str]) -> FormItem:
+        from fast_tmp.site import resources
+
         if not self._control:
             self._control = SelectItem(
                 name=self.name,
                 label=self.label,
-                source=f"get:{self._prefix}/select/{self.name}",
+                source=f"get:{self.prefix}/select/{self.name}",
                 labelField="label",
                 valueField="value",
             )
-            if not self._field.null:  # type: ignore
+            if not self._field.null:
                 self._control.required = True
             else:
                 self._control.clearable = True
+        if self.need_perms:
+            perm = self.need_perms[0]
+            if perm in codenames:
+                prefix = self._field.related_model.__name__.lower()
+                for model_prefix, model in resources.items():
+                    if prefix == model_prefix:
+                        controls = model.get_create_controls(request, codenames)
+                        self._control.creatable = True
+                        self._control.createBtnLabel = "新增"
+                        self._control.addControls = controls
+                        self._control.addApi = model.prefix + "/create"
+                        break
         return self._control
 
     def orm_2_amis(self, value: Any) -> Any:
@@ -335,9 +372,39 @@ class ForeignKeyControl(BaseAdminControl, RelationSelectApi):
         if value is not None:
             if isinstance(value, dict):
                 value = value.get("value")
-            value = await self._field.related_model.filter(pk=value).first()  # type: ignore
+            value = await self._field.related_model.filter(pk=value).first()
         setattr(obj, self.name, value)
 
+    def need_codenames(self, request: Request) -> Tuple[str, ...]:
+        """
+        判断，如果用户有对应页面的创建权限，则可以为该表增加创建按钮
+        """
+        if self.need_perms is None:
+            self.need_perms = (self.related_prefix() + "_create",)
+        return self.need_perms
+
+
+# 如果是可选，怎么过滤的问题？反向过滤？
+class OneToOneControl(ForeignKeyControl):
+    async def get_selects(
+        self,
+        request: Request,
+        pk: Optional[str],
+        perPage: Optional[int],
+        page: Optional[int],
+        filter: Any = None,
+    ):
+        exist_pks = await self.field.model.all().values(self.field.source_field)
+        field_model_all = self.field.related_model.filter(
+            **{self.field.to_field + "__not_in": [i[self.field.source_field] for i in exist_pks]}
+        )
+        data = await field_model_all
+        return {"options": [{"value": i.pk, "label": str(i)} for i in data]}
+
+    def get_column_inline(self, request: Request) -> Column:
+        # todo 一对一可以使用inline
+        raise AttributeError("foreignkey field can not be used in column inline.")
+
 
 class ManyToManyControl(BaseAdminControl, RelationSelectApi):
     _many = True
@@ -356,9 +423,7 @@ class ManyToManyControl(BaseAdminControl, RelationSelectApi):
                         dialog=Dialog(
                             title=self.label,
                             body=CRUD(
-                                api="get:"
-                                + self._field.model.__name__  # type: ignore
-                                + f"/select/{self.name}?pk=$pk",
+                                api=f"get:{self.prefix}/select/{self.name}?pk=$pk",
                                 columns=[
                                     Column(label="主键", name="pk"),
                                     Column(label="名称", name="label"),
@@ -378,16 +443,16 @@ class ManyToManyControl(BaseAdminControl, RelationSelectApi):
         page: Optional[int],
         filter: Any = None,
     ):
-        related_model = self._field.related_model  # type: ignore
+        related_model = self._field.related_model
         if pk is not None:
-            queryset = related_model.filter(**{self._field.related_name: pk})  # type: ignore
+            queryset = related_model.filter(**{self._field.related_name: pk})
         else:
             queryset = related_model.all()
-        if filter:
-            if isinstance(filter, dict):
-                queryset = queryset.filter(**filter)
-            else:
-                queryset = queryset(filter)
+        # if filter:
+        #     if isinstance(filter, dict):
+        #         queryset = queryset.filter(**filter)
+        #     else:
+        #         queryset = queryset(filter)
         if pk is not None:
             count = await queryset.count()
             data = await queryset
@@ -404,9 +469,9 @@ class ManyToManyControl(BaseAdminControl, RelationSelectApi):
             self._control = TransferItem(
                 name=self.name,
                 label=self.label,
-                source=f"get:{self._prefix}/select/{self.name}",
+                source=f"get:{self.prefix}/select/{self.name}",
             )
-            if self._field.null:  # type: ignore
+            if self._field.null:
                 self._control.clearable = True
         return self._control
 
@@ -415,11 +480,11 @@ class ManyToManyControl(BaseAdminControl, RelationSelectApi):
             return value.split(",")
         return [i["value"] for i in value]
 
-    async def validate(self, value: Any) -> Any:
+    async def validate(self, value: Any, is_create=False) -> Any:
         if value is not None:
             pks = self.amis_2_orm(value)
             if len(pks) > 0:
-                return await self._field.related_model.filter(pk__in=pks)  # type: ignore
+                return await self._field.related_model.filter(pk__in=pks)
         return []
 
     async def set_value(self, request: Request, obj: Model, value: Any) -> Optional[Coroutine]:
@@ -456,20 +521,156 @@ class ManyToManyControl(BaseAdminControl, RelationSelectApi):
         return [{"label": str(i), "value": i.pk} for i in value]
 
 
+# todo 增加创建按钮?
+class BackwardFKControl(ManyToManyControl):
+    _many = True
+    _control_type = FormItemEnum.select
+
+    def prefetch(self) -> Optional[str]:
+        return "prefetch"
+
+    def get_column(self, request: Request) -> Column:
+        if not self._column:
+            self._column = Operation(
+                label=self.label,
+                buttons=[
+                    DialogAction(
+                        label="查看",
+                        dialog=Dialog(
+                            title=self.label,
+                            body=CRUD(
+                                api=f"get:{self.prefix}/select/{self.name}?pk=$pk",
+                                columns=[
+                                    Column(label="主键", name="pk"),
+                                    Column(label="名称", name="label"),
+                                ],
+                            ),
+                        ),
+                    )
+                ],
+            )
+        return self._column
+
+    async def get_selects(
+        self,
+        request: Request,
+        pk: Optional[str],
+        perPage: Optional[int],
+        page: Optional[int],
+        filter: Any = None,
+    ):
+        related_model = self._field.related_model
+        if pk is not None:
+            queryset = related_model.filter(**{self._field.related_name: pk})
+        else:
+            queryset = related_model.all()
+        # if filter:
+        #     if isinstance(filter, dict):
+        #         queryset = queryset.filter(**filter)
+        #     else:
+        #         queryset = queryset(filter)
+        if pk is not None:
+            count = await queryset.count()
+            data = await queryset
+            return ListDataWithPage(
+                total=count,
+                items=[{"value": i.pk, "label": str(i)} for i in data],
+            )
+        else:
+            data = await queryset
+            return {"options": [{"value": i.pk, "label": str(i)} for i in data]}
+
+    def get_formitem(self, request: Request, codenames: Iterable[str]) -> FormItem:
+        raise AttributeError(f"BackwardFKControl field {self.name} can not be created")
+
+    def get_column_inline(self, request: Request) -> Column:
+        raise AttributeError(
+            f"BackwardFKControl field {self.name} can not be used in column inline."
+        )
+
+    def orm_2_amis(self, value: Any) -> Any:
+        return [{"label": str(i), "value": i.pk} for i in value]
+
+
+class BackwardOneToOneControl(OneToOneControl):
+    def get_column(self, request: Request) -> Column:
+        if not self._column:
+            self._column = Custom(
+                label=self.name,
+                name=self.name,
+                onMount=f"const text = document.createTextNode(value.label);dom.appendChild(text);${self.name}=value.value;",
+                onUpdate=f"const value=data.{self.name};dom.current.firstChild.textContent=value.label;${self.name}=value.value;",
+            )
+        return self._column
+
+    def get_formitem(self, request: Request, codenames: Iterable[str]) -> FormItem:
+        if not self._control:
+            self._control = SelectItem(
+                name=self.name,
+                label=self.label,
+                source=f"get:{self.prefix}/select/{self.name}",
+                labelField="label",
+                valueField="value",
+                clearable=True,
+            )
+        return self._control
+
+    def need_codenames(self, request: Request) -> Tuple[str, ...]:
+        """
+        判断，如果用户有对应页面的创建权限，则可以为该表增加创建按钮
+        """
+        if self.need_perms is None:
+            self.need_perms = (self.related_prefix() + "_create",)
+        return self.need_perms
+
+    async def get_selects(
+        self,
+        request: Request,
+        pk: Optional[str],
+        perPage: Optional[int],
+        page: Optional[int],
+        filter: Any = None,
+    ):
+        field_model_all = self.field.related_model.filter(**{self.field.relation_field: None})
+        # if filter:
+        #     if isinstance(filter, dict):
+        #         field_model_all = self._field.related_model.filter(**filter)
+        #     else:
+        #         field_model_all = self._field.related_model.filter(filter)
+        # if perPage is not None and page is not None:
+        #     field_model = field_model_all.limit(perPage).offset((page - 1) * perPage)
+        #     count = await field_model_all.count()
+        #     data = await field_model
+        #     return ListDataWithPage(
+        #         total=count,
+        #         items=[{"value": i.pk, "label": str(i)} for i in data],
+        #     )
+        # else:
+        data = await field_model_all
+        return {"options": [{"value": i.pk, "label": str(i)} for i in data]}
+
+    async def set_value(self, request: Request, obj: Model, value: Any):
+        """
+        一对一不允许修改
+        """
+        pass
+
+
 class FileControl(BaseAdminControl):
     _control_type = FormItemEnum.input_file
+    prefix = ""
 
     def get_formitem(self, request: Request, codenames: Iterable[str]) -> FormItem:
         if not self._control:
             self._control = FileItem(
                 name=self.name,
                 label=self.label,
-                receiver=f"{self._field.model.__name__}/file/{self.name}",  # type: ignore
+                receiver=f"{self.prefix}/file/{self.name}",
             )
-            if not self._field.null:  # type: ignore
+            if not self._field.null:
                 self._control.required = True
-            if self._field.default is not None:  # type: ignore
-                self._control.value = self.orm_2_amis(self._field.default)  # type: ignore
+            if self._field.default is not None:
+                self._control.value = self.orm_2_amis(self._field.default)
         return self._control
 
     def orm_2_amis(self, value: str) -> Any:
@@ -483,18 +684,19 @@ class FileControl(BaseAdminControl):
 
 class ImageControl(BaseAdminControl):
     _control_type = FormItemEnum.input_image
+    prefix = ""
 
     def get_formitem(self, request: Request, codenames: Iterable[str]) -> FormItem:
         if not self._control:
             self._control = ImageItem(
                 name=self.name,
                 label=self.label,
-                receiver=f"{self._field.model.__name__}/file/{self.name}",  # type: ignore
+                receiver=f"{self.prefix}/file/{self.name}",
             )
-            if not self._field.null:  # type: ignore
+            if not self._field.null:
                 self._control.required = True
-            if self._field.default is not None:  # type: ignore
-                self._control.value = self.orm_2_amis(self._field.default)  # type: ignore
+            if self._field.default is not None:
+                self._control.value = self.orm_2_amis(self._field.default)
         return self._control
 
     def get_column(self, request: Request) -> Column:
@@ -517,10 +719,10 @@ class RichTextControl(BaseAdminControl):
     def get_formitem(self, request: Request, codenames: Iterable[str]) -> FormItem:
         if not self._control:
             self._control = RichTextItem(type=self._control_type, name=self.name, label=self.label)
-            if not self._field.null:  # type: ignore
+            if not self._field.null:
                 self._control.required = True
-            if self._field.default is not None:  # type: ignore
-                self._control.value = self.orm_2_amis(self._field.default)  # type: ignore
+            if self._field.default is not None:
+                self._control.value = self.orm_2_amis(self._field.default)
         return self._control
 
 
@@ -530,46 +732,68 @@ def create_column(
     prefix: str,
 ):
     if isinstance(field_type, IntEnumFieldInstance):
-        return IntEnumControl(field_name, field_type, prefix)
+        return IntEnumControl("", field_name, field_type.null, field_type.default, field=field_type)
     elif isinstance(field_type, CharEnumFieldInstance):
-        return StrEnumControl(field_name, field_type, prefix)
+        return StrEnumControl("", field_name, field_type.null, field_type.default, field=field_type)
     elif isinstance(
         field_type, (fields.IntField, fields.SmallIntField, fields.BigIntField, fields.FloatField)
     ):
-        return IntControl(field_name, field_type, prefix)
+        return IntControl("", field_name, field_type.null, field_type.default, field=field_type)
     elif isinstance(field_type, fields.TextField):
-        return TextControl(field_name, field_type, prefix)
+        return TextControl("", field_name, field_type.null, field_type.default, field=field_type)
     elif isinstance(field_type, fields.DatetimeField):
-        return DateTimeControl(field_name, field_type, prefix)
+        return DateTimeControl(
+            "", field_name, field_type.null, field_type.default, field=field_type
+        )
     elif isinstance(field_type, fields.DateField):
-        return DateControl(field_name, field_type, prefix)
+        return DateControl("", field_name, field_type.null, field_type.default, field=field_type)
     elif isinstance(field_type, fields.CharField):
-        return StrControl(field_name, field_type, prefix)
+        return StrControl("", field_name, field_type.null, field_type.default, field=field_type)
     elif isinstance(field_type, fields.BooleanField):
-        return BooleanControl(field_name, field_type, prefix)
+        return BooleanControl("", field_name, field_type.null, field_type.default, field=field_type)
     elif isinstance(field_type, fields.JSONField):
-        return JsonControl(field_name, field_type, prefix)
+        return JsonControl("", field_name, field_type.null, field_type.default, field=field_type)
     elif isinstance(field_type, fields.TimeField):
-        return TimeControl(field_name, field_type, prefix)
-    elif isinstance(field_type, ForeignKeyFieldInstance):
-        return ForeignKeyControl(field_name, field_type, prefix)
+        return TimeControl("", field_name, field_type.null, field_type.default, field=field_type)
     elif isinstance(field_type, ManyToManyFieldInstance):
-        return ManyToManyControl(field_name, field_type, prefix)
-    elif isinstance(field_type, ImageField):
-        return ImageControl(field_name, field_type, prefix)
-    elif isinstance(field_type, RichTextField):
-        return RichTextControl(field_name, field_type, prefix)
-    elif isinstance(field_type, FileField):
-        return FileControl(field_name, field_type, prefix)
-    elif isinstance(field_type, fields.DecimalField):
-        return DecimalControl(field_name, field_type, prefix)
+        return ManyToManyControl(
+            "", field_name, field_type.null, field_type.default, field=field_type, prefix=prefix
+        )
+    elif isinstance(field_type, OneToOneFieldInstance):
+        return OneToOneControl(
+            "", field_name, field_type.null, field_type.default, field=field_type, prefix=prefix
+        )
+    elif isinstance(field_type, ForeignKeyFieldInstance):
+        return ForeignKeyControl(
+            "", field_name, field_type.null, field_type.default, field=field_type, prefix=prefix
+        )
+    elif isinstance(field_type, BackwardOneToOneRelation):
+        return BackwardOneToOneControl(
+            "", field_name, field_type.null, field_type.default, field=field_type, prefix=prefix
+        )
+    elif isinstance(field_type, BackwardFKRelation):  # 多对一反向
+        return BackwardFKControl(
+            "", field_name, field_type.null, field_type.default, field=field_type, prefix=prefix
+        )
 
+    elif isinstance(field_type, ImageField):
+        return ImageControl(
+            "", field_name, field_type.null, field_type.default, field=field_type, prefix=prefix
+        )
+    elif isinstance(field_type, RichTextField):
+        return RichTextControl(
+            "", field_name, field_type.null, field_type.default, field=field_type
+        )
+    elif isinstance(field_type, FileField):
+        return FileControl("", field_name, field_type.null, field_type.default, field=field_type)
+    elif isinstance(field_type, fields.DecimalField):
+        return DecimalControl("", field_name, field_type.null, field_type.default, field=field_type)
     else:
         raise AmisStructError("create_column error:", type(field_type))
 
 
-# todo 以后考虑创建更新的control分离
-class Password(StrControl):
+# fixme 以后考虑创建更新的control分离?
+class Password(BaseControl):
     _control_type = FormItemEnum.input_password
 
     async def get_value(self, request: Request, obj: Model) -> Any:
@@ -588,7 +812,7 @@ class Password(StrControl):
     def get_formitem(self, request: Request, codenames: Iterable[str]) -> FormItem:
         if not self._control:
             self._control = FormItem(type=self._control_type, name=self.name, label=self.label)
-            if not self._field.null:  # type: ignore
-                if self._field.default is not None:  # type: ignore
-                    self._control.value = self.orm_2_amis(self._field.default)  # type: ignore
+            if not self.null:
+                if self.default is not None:
+                    self._control.value = self.orm_2_amis(self.default)
         return self._control
